#include "ConstantTable.h"

namespace ShaderConst {

// Shader instruction opcodes
const uint32_t SIO_COMMENT = 0x0000FFFE;
const uint32_t SIO_END = 0x0000FFFF;
const uint32_t SI_OPCODE_MASK = 0x0000FFFF;
const uint32_t SI_COMMENTSIZE_MASK = 0x7FFF0000;
const uint32_t CTAB_CONSTANT = 0x42415443;

// Member functions
bool ConstantTable::Create(const void* data)
{
  const uint32_t* ptr = static_cast<const uint32_t*>(data);
  while(*++ptr != SIO_END)
  {
    if((*ptr & SI_OPCODE_MASK) == SIO_COMMENT)
    {
      // Check for CTAB comment
      uint32_t comment_size = (*ptr & SI_COMMENTSIZE_MASK) >> 16;
      if(*(ptr+1) != CTAB_CONSTANT)
      {
        ptr += comment_size;
        continue;
      }

      // Read header
      const char* ctab = reinterpret_cast<const char*>(ptr+2);
      size_t ctab_size = (comment_size-1)*4;

      const CTHeader* header = reinterpret_cast<const CTHeader*>(ctab);
      if(ctab_size < sizeof(*header) || header->Size != sizeof(*header))
        return false;
      m_creator = ctab + header->Creator;

      // Read constants
      m_constants.reserve(header->Constants);
      const CTInfo* info = reinterpret_cast<const CTInfo*>(ctab + header->ConstantInfo);
      for(uint32_t i = 0; i < header->Constants; ++i)
      {
        const CTType* type = reinterpret_cast<const CTType*>(ctab + info[i].TypeInfo);

        // Fill struct
        ConstantDesc desc;
        desc.Name = ctab + info[i].Name;
        desc.RegisterSet = static_cast<EREGISTER_SET>(info[i].RegisterSet);
        desc.RegisterIndex = info[i].RegisterIndex;
        desc.RegisterCount = info[i].RegisterCount;
        desc.Class = type->Class;
        desc.Rows = type->Rows;
        desc.Columns = type->Columns;
        desc.Elements = type->Elements;
        desc.StructMembers = type->StructMembers;
        desc.Bytes = 4 * desc.Elements * desc.Rows * desc.Columns;
        m_constants.push_back(desc);
        
        printf("name=%s RI=%d, RC=%d\n",
        	desc.Name.c_str(), desc.RegisterIndex, desc.RegisterCount);
        
      }
      return true;
    }
  }
  return false;
}

const ConstantDesc* ConstantTable::GetConstantByName(const std::string& name) const
{
  std::vector<ConstantDesc>::const_iterator it;
  for(it = m_constants.begin(); it != m_constants.end(); ++it)
  {
    if(it->Name == name)
      return &(*it);
  }
  return NULL;
}

}
